# encoding: utf-8
"""
These are almost end-to-end tests. They create a CommandLineInterface
instance, feed it with some input and check the result.
"""
from __future__ import unicode_literals
from prompt_toolkit.application import Application
from prompt_toolkit.enums import DEFAULT_BUFFER, EditingMode
from prompt_toolkit.eventloop.posix import PosixEventLoop
from prompt_toolkit.input import PipeInput
from prompt_toolkit.interface import CommandLineInterface
from prompt_toolkit.output import DummyOutput
from functools import partial


def _feed_cli_with_input(text, editing_mode=EditingMode.EMACS):
    """
    Create a CommandLineInterface, feed it with the given user input and return
    the CLI object.

    This returns a (result, CLI) tuple.
    """
    # If the given text doesn't end with a newline, the interface won't finish.
    assert text.endswith('\n')

    loop = PosixEventLoop()
    try:
        inp = PipeInput()
        inp.send_text(text)
        cli = CommandLineInterface(
            application=Application(editing_mode=editing_mode),
            eventloop=loop,
            input=inp,
            output=DummyOutput())
        result = cli.run()
        return result, cli
    finally:
        loop.close()
        inp.close()


def test_simple_text_input():
    # Simple text input, followed by enter.
    result, cli = _feed_cli_with_input('hello\n')
    assert result.text == 'hello'
    assert cli.buffers[DEFAULT_BUFFER].text == 'hello'


def test_emacs_cursor_movements():
    """
    Test cursor movements with Emacs key bindings.
    """
    # ControlA
    result, cli = _feed_cli_with_input('hello\x01X\n')
    assert result.text == 'Xhello'

    # ControlH or \b
    result, cli = _feed_cli_with_input('hello\x08X\n')
    assert result.text == 'hellX'

    # Delete.  (Left, left, delete)
    result, cli = _feed_cli_with_input('hello\x1b[D\x1b[D\x1b[3~\n')
    assert result.text == 'helo'

    # Left.
    result, cli = _feed_cli_with_input('hello\x1b[DX\n')
    assert result.text == 'hellXo'

    # ControlA, right
    result, cli = _feed_cli_with_input('hello\x01\x1b[CX\n')
    assert result.text == 'hXello'

    # ControlA, right
    result, cli = _feed_cli_with_input('hello\x01\x1b[CX\n')
    assert result.text == 'hXello'

    # ControlB (Emacs cursor left.)
    result, cli = _feed_cli_with_input('hello\x02X\n')
    assert result.text == 'hellXo'

    # ControlC: ignored by default, unless the prompt-bindings are loaded.
    result, cli = _feed_cli_with_input('hello\x03\n')
    assert result.text == 'hello'

    # ControlD: ignored by default, unless the prompt-bindings are loaded.
    result, cli = _feed_cli_with_input('hello\x04\n')
    assert result.text == 'hello'

    # Left, Left, ControlK
    result, cli = _feed_cli_with_input('hello\x1b[D\x1b[D\x0b\n')
    assert result.text == 'hel'

    # ControlL: should not influence the result.
    result, cli = _feed_cli_with_input('hello\x0c\n')
    assert result.text == 'hello'


def test_emacs_other_bindings():
    # Transpose characters.
    result, cli = _feed_cli_with_input('abcde\x14X\n')  # Ctrl-T
    assert result.text == 'abcedX'

    # Left, Left, Transpose. (This is slightly different.)
    result, cli = _feed_cli_with_input('abcde\x1b[D\x1b[D\x14X\n')
    assert result.text == 'abdcXe'

    # Clear before cursor.
    result, cli = _feed_cli_with_input('hello\x1b[D\x1b[D\x15X\n')
    assert result.text == 'Xlo'

    # Delete word before the cursor.
    result, cli = _feed_cli_with_input('hello world test\x17X\n')
    assert result.text == 'hello world X'

    # (with argument.)
    result, cli = _feed_cli_with_input('hello world test\x1b2\x17X\n')
    assert result.text == 'hello X'


def test_vi_cursor_movements():
    """
    Test cursor movements with Vi key bindings.
    """
    feed = partial(_feed_cli_with_input, editing_mode=EditingMode.VI)

    result, cli = feed('\x1b\n')
    assert result.text == ''
    assert cli.editing_mode == EditingMode.VI

    # Esc h a X
    result, cli = feed('hello\x1bhaX\n')
    assert result.text == 'hellXo'

    # Esc I X
    result, cli = feed('hello\x1bIX\n')
    assert result.text == 'Xhello'

    # Esc I X
    result, cli = feed('hello\x1bIX\n')
    assert result.text == 'Xhello'

    # Esc 2hiX
    result, cli = feed('hello\x1b2hiX\n')
    assert result.text == 'heXllo'

    # Esc 2h2liX
    result, cli = feed('hello\x1b2h2liX\n')
    assert result.text == 'hellXo'

    # Esc \b\b
    result, cli = feed('hello\b\b\n')
    assert result.text == 'hel'

    # Esc \b\b
    result, cli = feed('hello\b\b\n')
    assert result.text == 'hel'

    # Esc 2h D
    result, cli = feed('hello\x1b2hD\n')
    assert result.text == 'he'

    # Esc 2h rX \n
    result, cli = feed('hello\x1b2hrX\n')
    assert result.text == 'heXlo'


def test_vi_operators():
    feed = partial(_feed_cli_with_input, editing_mode=EditingMode.VI)

    # Esc g~0
    result, cli = feed('hello\x1bg~0\n')
    assert result.text == 'HELLo'

    # Esc gU0
    result, cli = feed('hello\x1bgU0\n')
    assert result.text == 'HELLo'

    # Esc d0
    result, cli = feed('hello\x1bd0\n')
    assert result.text == 'o'


def test_vi_text_objects():
    feed = partial(_feed_cli_with_input, editing_mode=EditingMode.VI)

    # Esc gUgg
    result, cli = feed('hello\x1bgUgg\n')
    assert result.text == 'HELLO'

    # Esc gUU
    result, cli = feed('hello\x1bgUU\n')
    assert result.text == 'HELLO'

    # Esc di(
    result, cli = feed('before(inside)after\x1b8hdi(\n')
    assert result.text == 'before()after'

    # Esc di[
    result, cli = feed('before[inside]after\x1b8hdi[\n')
    assert result.text == 'before[]after'

    # Esc da(
    result, cli = feed('before(inside)after\x1b8hda(\n')
    assert result.text == 'beforeafter'


def test_vi_digraphs():
    feed = partial(_feed_cli_with_input, editing_mode=EditingMode.VI)

    # C-K o/
    result, cli = feed('hello\x0bo/\n')
    assert result.text == 'helloø'

    # C-K e:
    result, cli = feed('hello\x0be:\n')
    assert result.text == 'helloë'



#!/usr/bin/env python
"""
Simple example of input validation.
"""
from __future__ import unicode_literals

from prompt_toolkit.validation import Validator, ValidationError
from prompt_toolkit import prompt


class EmailValidator(Validator):
    def validate(self, document):
        if '@' not in document.text:
            raise ValidationError(message='Not a valid e-mail address (Does not contain an @).',
                                  cursor_position=len(document.text))  # Move cursor to end of input.


def main():
    text = prompt('Enter e-mail address: ', validator=EmailValidator())
    print('You said: %s' % text)

if __name__ == '__main__':
    main()



from __future__ import unicode_literals
from prompt_toolkit.cache import SimpleCache
from prompt_toolkit.filters import CLIFilter, to_cli_filter, Never
from prompt_toolkit.keys import Key, Keys

from six import text_type

__all__ = (
    'Registry',
)


class _Binding(object):
    """
    (Immutable binding class.)
    """
    def __init__(self, keys, handler, filter=None, eager=None, save_before=None):
        assert isinstance(keys, tuple)
        assert callable(handler)
        assert isinstance(filter, CLIFilter)
        assert isinstance(eager, CLIFilter)
        assert callable(save_before)

        self.keys = keys
        self.handler = handler
        self.filter = filter
        self.eager = eager
        self.save_before = save_before

    def call(self, event):
        return self.handler(event)

    def __repr__(self):
        return '%s(keys=%r, handler=%r)' % (
            self.__class__.__name__, self.keys, self.handler)


class Registry(object):
    """
    Key binding registry.

    ::

        r = Registry()

        @r.add_binding(Keys.ControlX, Keys.ControlC, filter=INSERT)
        def handler(event):
            # Handle ControlX-ControlC key sequence.
            pass
    """
    def __init__(self):
        self.key_bindings = []
        self._get_bindings_for_keys_cache = SimpleCache(maxsize=10000)
        self._get_bindings_starting_with_keys_cache = SimpleCache(maxsize=1000)

    def _clear_cache(self):
        self._get_bindings_for_keys_cache.clear()
        self._get_bindings_starting_with_keys_cache.clear()

    def add_binding(self, *keys, **kwargs):
        """
        Decorator for annotating key bindings.

        :param filter: :class:`~prompt_toolkit.filters.CLIFilter` to determine
            when this key binding is active.
        :param eager: :class:`~prompt_toolkit.filters.CLIFilter` or `bool`.
            When True, ignore potential longer matches when this key binding is
            hit. E.g. when there is an active eager key binding for Ctrl-X,
            execute the handler immediately and ignore the key binding for
            Ctrl-X Ctrl-E of which it is a prefix.
        :param save_before: Callable that takes an `Event` and returns True if
            we should save the current buffer, before handling the event.
            (That's the default.)
        """
        filter = to_cli_filter(kwargs.pop('filter', True))
        eager = to_cli_filter(kwargs.pop('eager', False))
        save_before = kwargs.pop('save_before', lambda e: True)
        to_cli_filter(kwargs.pop('invalidate_ui', True))  # Deprecated! (ignored.)

        assert not kwargs
        assert keys
        assert all(isinstance(k, (Key, text_type)) for k in keys), \
            'Key bindings should consist of Key and string (unicode) instances.'
        assert callable(save_before)

        def decorator(func):
            # When a filter is Never, it will always stay disabled, so in that case
            # don't bother putting it in the registry. It will slow down every key
            # press otherwise.
            if not isinstance(filter, Never):
                self.key_bindings.append(
                    _Binding(keys, func, filter=filter, eager=eager,
                             save_before=save_before))
                self._clear_cache()

            return func
        return decorator

    def remove_binding(self, function):
        """
        Remove a key binding.

        This expects a function that was given to `add_binding` method as
        parameter. Raises `ValueError` when the given function was not
        registered before.
        """
        assert callable(function)

        for b in self.key_bindings:
            if b.handler == function:
                self.key_bindings.remove(b)
                self._clear_cache()
                return

        # No key binding found for this function. Raise ValueError.
        raise ValueError('Binding not found: %r' % (function, ))

    def get_bindings_for_keys(self, keys):
        """
        Return a list of key bindings that can handle this key.
        (This return also inactive bindings, so the `filter` still has to be
        called, for checking it.)

        :param keys: tuple of keys.
        """
        def get():
            result = []
            for b in self.key_bindings:
                if len(keys) == len(b.keys):
                    match = True
                    any_count = 0

                    for i, j in zip(b.keys, keys):
                        if i != j and i != Keys.Any:
                            match = False
                            break

                        if i == Keys.Any:
                            any_count += 1

                    if match:
                        result.append((any_count, b))

            # Place bindings that have more 'Any' occurences in them at the end.
            result = sorted(result, key=lambda item: -item[0])

            return [item[1] for item in result]

        return self._get_bindings_for_keys_cache.get(keys, get)

    def get_bindings_starting_with_keys(self, keys):
        """
        Return a list of key bindings that handle a key sequence starting with
        `keys`. (It does only return bindings for which the sequences are
        longer than `keys`. And like `get_bindings_for_keys`, it also includes
        inactive bindings.)

        :param keys: tuple of keys.
        """
        def get():
            result = []
            for b in self.key_bindings:
                if len(keys) < len(b.keys):
                    match = True
                    for i, j in zip(b.keys, keys):
                        if i != j and i != Keys.Any:
                            match = False
                            break
                    if match:
                        result.append(b)
            return result

        return self._get_bindings_starting_with_keys_cache.get(keys, get)



from __future__ import unicode_literals
from prompt_toolkit.filters import CLIFilter, Always

__all__ = (
    'create_handle_decorator',
)

def create_handle_decorator(registry, filter=Always()):
    """
    Create a key handle decorator, which is compatible with `Registry.handle`,
    but will chain the given filter to every key binding.

    :param filter: `CLIFilter`
    """
    assert isinstance(filter, CLIFilter)

    def handle(*keys, **kw):
        # Chain the given filter to the filter of this specific binding.
        if 'filter' in kw:
            kw['filter'] = kw['filter'] & filter
        else:
            kw['filter'] = filter

        return registry.add_binding(*keys, **kw)
    return handle



"""
The default styling.
"""
from __future__ import unicode_literals

from prompt_toolkit.token import Token

__all__ = (
    'DEFAULT_STYLE_EXTENSIONS',
    'default_style_extensions',
)


#: Styling of prompt-toolkit specific tokens, that are not know by the default
#: Pygments style.
DEFAULT_STYLE_EXTENSIONS = {
    # Highlighting of search matches in document.
    Token.SearchMatch:                            'noinherit reverse',
    Token.SearchMatch.Current:                    'noinherit #ffffff bg:#448844 underline',

    # Highlighting of select text in document.
    Token.SelectedText:                           'reverse',

    Token.CursorColumn:                           'bg:#dddddd',
    Token.CursorLine:                             'underline',
    Token.ColorColumn:                            'bg:#ccaacc',

    # Highlighting of matching brackets.
    Token.MatchingBracket:                        '',
    Token.MatchingBracket.Other:                  '#000000 bg:#aacccc',
    Token.MatchingBracket.Cursor:                 '#ff8888 bg:#880000',

    # Line numbers.
    Token.LineNumber:                             '#888888',
    Token.LineNumber.Current:                     'bold',
    Token.Tilde:                                  '#8888ff',

    # Default prompt.
    Token.Prompt:                                 '',
    Token.Prompt.Arg:                             'noinherit',
    Token.Prompt.Search:                          'noinherit',
    Token.Prompt.Search.Text:                     '',

    # Search toolbar.
    Token.Toolbar.Search:                         'bold',
    Token.Toolbar.Search.Text:                    'nobold',

    # System toolbar
    Token.Toolbar.System:                         'bold',
    Token.Toolbar.System.Text:                    'nobold',

    # "arg" toolbar.
    Token.Toolbar.Arg:                            'bold',
    Token.Toolbar.Arg.Text:                       'nobold',

    # Validation toolbar.
    Token.Toolbar.Validation:                     'bg:#550000 #ffffff',
    Token.WindowTooSmall:                         'bg:#550000 #ffffff',

    # Completions toolbar.
    Token.Toolbar.Completions:                    'bg:#bbbbbb #000000',
    Token.Toolbar.Completions.Arrow:              'bg:#bbbbbb #000000 bold',
    Token.Toolbar.Completions.Completion:         'bg:#bbbbbb #000000',
    Token.Toolbar.Completions.Completion.Current: 'bg:#444444 #ffffff',

    # Completions menu.
    Token.Menu.Completions:                       'bg:#bbbbbb #000000',
    Token.Menu.Completions.Completion:            '',
    Token.Menu.Completions.Completion.Current:    'bg:#888888 #ffffff',
    Token.Menu.Completions.Meta:                  'bg:#999999 #000000',
    Token.Menu.Completions.Meta.Current:          'bg:#aaaaaa #000000',
    Token.Menu.Completions.MultiColumnMeta:       'bg:#aaaaaa #000000',

    # Scrollbars.
    Token.Scrollbar:                              'bg:#888888',
    Token.Scrollbar.Button:                       'bg:#444444',
    Token.Scrollbar.Arrow:                        'bg:#222222 #888888 bold',

    # Auto suggestion text.
    Token.AutoSuggestion:                         '#666666',

    # Trailing whitespace and tabs.
    Token.TrailingWhiteSpace:                     '#999999',
    Token.Tab:                                    '#999999',

    # When Control-C has been pressed. Grayed.
    Token.Aborted:                                '#888888',

    # Entering a Vi digraph.
    Token.Digraph:                                '#4444ff',
}

default_style_extensions = DEFAULT_STYLE_EXTENSIONS  # Old name.



from __future__ import unicode_literals

from .buffer import Buffer, AcceptAction
from .buffer_mapping import BufferMapping
from .clipboard import Clipboard, InMemoryClipboard
from .enums import DEFAULT_BUFFER, EditingMode
from .filters import CLIFilter, to_cli_filter
from .key_binding.bindings.basic import load_basic_bindings
from .key_binding.bindings.emacs import load_emacs_bindings
from .key_binding.bindings.vi import load_vi_bindings
from .key_binding.registry import Registry
from .layout import Window
from .layout.containers import Container
from .layout.controls import BufferControl
from .styles import DEFAULT_STYLE, Style
import six

__all__ = (
    'AbortAction',
    'Application',
)


class AbortAction(object):
    """
    Actions to take on an Exit or Abort exception.
    """
    RETRY = 'retry'
    RAISE_EXCEPTION = 'raise-exception'
    RETURN_NONE = 'return-none'

    _all = (RETRY, RAISE_EXCEPTION, RETURN_NONE)


class Application(object):
    """
    Application class to be passed to a
    :class:`~prompt_toolkit.interface.CommandLineInterface`.

    This contains all customizable logic that is not I/O dependent.
    (So, what is independent of event loops, input and output.)

    This way, such an :class:`.Application` can run easily on several
    :class:`~prompt_toolkit.interface.CommandLineInterface` instances, each
    with a different I/O backends. that runs for instance over telnet, SSH or
    any other I/O backend.

    :param layout: A :class:`~prompt_toolkit.layout.containers.Container` instance.
    :param buffer: A :class:`~prompt_toolkit.buffer.Buffer` instance for the default buffer.
    :param initial_focussed_buffer: Name of the buffer that is focussed during start-up.
    :param key_bindings_registry:
        :class:`~prompt_toolkit.key_binding.registry.Registry` instance for the
        key bindings.
    :param clipboard: :class:`~prompt_toolkit.clipboard.base.Clipboard` to use.
    :param on_abort: What to do when Control-C is pressed.
    :param on_exit: What to do when Control-D is pressed.
    :param use_alternate_screen: When True, run the application on the alternate screen buffer.
    :param get_title: Callable that returns the current title to be displayed in the terminal.
    :param erase_when_done: (bool) Clear the application output when it finishes.

    Filters:

    :param mouse_support: (:class:`~prompt_toolkit.filters.CLIFilter` or
        boolean). When True, enable mouse support.
    :param paste_mode: :class:`~prompt_toolkit.filters.CLIFilter` or boolean.
    :param ignore_case: :class:`~prompt_toolkit.filters.CLIFilter` or boolean.
    :param editing_mode: :class:`~prompt_toolkit.enums.EditingMode`.

    Callbacks (all of these should accept a
    :class:`~prompt_toolkit.interface.CommandLineInterface` object as input.)

    :param on_input_timeout: Called when there is no input for x seconds.
                    (Fired when any eventloop.onInputTimeout is fired.)
    :param on_start: Called when reading input starts.
    :param on_stop: Called when reading input ends.
    :param on_reset: Called during reset.
    :param on_buffer_changed: Called when the content of a buffer has been changed.
    :param on_initialize: Called after the
        :class:`~prompt_toolkit.interface.CommandLineInterface` initializes.
    :param on_render: Called right after rendering.
    :param on_invalidate: Called when the UI has been invalidated.
    """
    def __init__(self, layout=None, buffer=None, buffers=None,
                 initial_focussed_buffer=DEFAULT_BUFFER,
                 style=None,
                 key_bindings_registry=None, clipboard=None,
                 on_abort=AbortAction.RAISE_EXCEPTION, on_exit=AbortAction.RAISE_EXCEPTION,
                 use_alternate_screen=False, mouse_support=False,
                 get_title=None,

                 paste_mode=False, ignore_case=False, editing_mode=EditingMode.EMACS,
                 erase_when_done=False,

                 on_input_timeout=None, on_start=None, on_stop=None,
                 on_reset=None, on_initialize=None, on_buffer_changed=None,
                 on_render=None, on_invalidate=None):

        paste_mode = to_cli_filter(paste_mode)
        ignore_case = to_cli_filter(ignore_case)
        mouse_support = to_cli_filter(mouse_support)

        assert layout is None or isinstance(layout, Container)
        assert buffer is None or isinstance(buffer, Buffer)
        assert buffers is None or isinstance(buffers, (dict, BufferMapping))
        assert key_bindings_registry is None or isinstance(key_bindings_registry, Registry)
        assert clipboard is None or isinstance(clipboard, Clipboard)
        assert on_abort in AbortAction._all
        assert on_exit in AbortAction._all
        assert isinstance(use_alternate_screen, bool)
        assert get_title is None or callable(get_title)
        assert isinstance(paste_mode, CLIFilter)
        assert isinstance(ignore_case, CLIFilter)
        assert isinstance(editing_mode, six.string_types)
        assert on_input_timeout is None or callable(on_input_timeout)
        assert style is None or isinstance(style, Style)
        assert isinstance(erase_when_done, bool)

        assert on_start is None or callable(on_start)
        assert on_stop is None or callable(on_stop)
        assert on_reset is None or callable(on_reset)
        assert on_buffer_changed is None or callable(on_buffer_changed)
        assert on_initialize is None or callable(on_initialize)
        assert on_render is None or callable(on_render)
        assert on_invalidate is None or callable(on_invalidate)

        self.layout = layout or Window(BufferControl())

        # Make sure that the 'buffers' dictionary is a BufferMapping.
        # NOTE: If no buffer is given, we create a default Buffer, with IGNORE as
        #       default accept_action. This is what makes sense for most users
        #       creating full screen applications. Doing nothing is the obvious
        #       default. Those creating a REPL would use the shortcuts module that
        #       passes in RETURN_DOCUMENT.
        self.buffer = buffer or Buffer(accept_action=AcceptAction.IGNORE)
        if not buffers or not isinstance(buffers, BufferMapping):
            self.buffers = BufferMapping(buffers, initial=initial_focussed_buffer)
        else:
            self.buffers = buffers

        if buffer:
            self.buffers[DEFAULT_BUFFER] = buffer

        self.initial_focussed_buffer = initial_focussed_buffer

        self.style = style or DEFAULT_STYLE

        if key_bindings_registry is None:
            key_bindings_registry = Registry()
            load_basic_bindings(key_bindings_registry)
            load_emacs_bindings(key_bindings_registry)
            load_vi_bindings(key_bindings_registry)

        if get_title is None:
            get_title = lambda: None

        self.key_bindings_registry = key_bindings_registry
        self.clipboard = clipboard or InMemoryClipboard()
        self.on_abort = on_abort
        self.on_exit = on_exit
        self.use_alternate_screen = use_alternate_screen
        self.mouse_support = mouse_support
        self.get_title = get_title

        self.paste_mode = paste_mode
        self.ignore_case = ignore_case
        self.editing_mode = editing_mode
        self.erase_when_done = erase_when_done

        def dummy_handler(cli):
            " Dummy event handler. "

        self.on_input_timeout = on_input_timeout or dummy_handler
        self.on_start = on_start or dummy_handler
        self.on_stop = on_stop or dummy_handler
        self.on_reset = on_reset or dummy_handler
        self.on_initialize = on_initialize or dummy_handler
        self.on_buffer_changed = on_buffer_changed or dummy_handler
        self.on_render = on_render or dummy_handler
        self.on_invalidate = on_invalidate or dummy_handler





#real_property = property
#
#def property(func):
#    name = '_cache_' + func.__name__
#    def wrapper(self, *a, **kw):
#        if hasattr(self, name):
#            value = getattr(self, name) + 1
#        else:
#            value = 1
#        setattr(self, name, value)
#
#        if True:#value > 1:
#            with open('/tmp/log', 'a') as f:
#                f.write('accessed %r %i times\n' % (func.__name__, value))
#
#        return func(self, *a, **kw)
#
#    return real_property(wrapper)



from __future__ import unicode_literals

from prompt_toolkit.buffer import Buffer

import pytest


@pytest.fixture
def _buffer():
    return Buffer()


def test_initial(_buffer):
    assert _buffer.text == ''
    assert _buffer.cursor_position == 0


def test_insert_text(_buffer):
    _buffer.insert_text('some_text')
    assert _buffer.text == 'some_text'
    assert _buffer.cursor_position == len('some_text')


def test_cursor_movement(_buffer):
    _buffer.insert_text('some_text')
    _buffer.cursor_left()
    _buffer.cursor_left()
    _buffer.cursor_left()
    _buffer.cursor_right()
    _buffer.insert_text('A')

    assert _buffer.text == 'some_teAxt'
    assert _buffer.cursor_position == len('some_teA')


def test_backspace(_buffer):
    _buffer.insert_text('some_text')
    _buffer.cursor_left()
    _buffer.cursor_left()
    _buffer.delete_before_cursor()

    assert _buffer.text == 'some_txt'
    assert _buffer.cursor_position == len('some_t')


def test_cursor_up(_buffer):
    # Cursor up to a line thats longer.
    _buffer.insert_text('long line1\nline2')
    _buffer.cursor_up()

    assert _buffer.document.cursor_position == 5

    # Going up when already at the top.
    _buffer.cursor_up()
    assert _buffer.document.cursor_position == 5

    # Going up to a line that's shorter.
    _buffer.reset()
    _buffer.insert_text('line1\nlong line2')

    _buffer.cursor_up()
    assert _buffer.document.cursor_position == 5


def test_cursor_down(_buffer):
    _buffer.insert_text('line1\nline2')
    _buffer.cursor_position = 3

    # Normally going down
    _buffer.cursor_down()
    assert _buffer.document.cursor_position == len('line1\nlin')

    # Going down to a line that's storter.
    _buffer.reset()
    _buffer.insert_text('long line1\na\nb')
    _buffer.cursor_position = 3

    _buffer.cursor_down()
    assert _buffer.document.cursor_position == len('long line1\na')


def test_join_next_line(_buffer):
    _buffer.insert_text('line1\nline2\nline3')
    _buffer.cursor_up()
    _buffer.join_next_line()

    assert _buffer.text == 'line1\nline2 line3'

    # Test when there is no '\n' in the text
    _buffer.reset()
    _buffer.insert_text('line1')
    _buffer.cursor_position = 0
    _buffer.join_next_line()

    assert _buffer.text == 'line1'


def test_newline(_buffer):
    _buffer.insert_text('hello world')
    _buffer.newline()

    assert _buffer.text == 'hello world\n'


def test_swap_characters_before_cursor(_buffer):
    _buffer.insert_text('hello world')
    _buffer.cursor_left()
    _buffer.cursor_left()
    _buffer.swap_characters_before_cursor()

    assert _buffer.text == 'hello wrold'



#!/usr/bin/env python
from __future__ import unicode_literals
from prompt_toolkit import prompt


if __name__ == '__main__':
    password = prompt('Password: ', is_password=True)
    print('You said: %s' % password)



from __future__ import unicode_literals
from prompt_toolkit.validation import Validator, ValidationError
from six import string_types


class SentenceValidator(Validator):
    """
    Validate input only when it appears in this list of sentences.

    :param sentences: List of sentences.
    :param ignore_case: If True, case-insensitive comparisons.
    """
    def __init__(self, sentences, ignore_case=False, error_message='Invalid input', move_cursor_to_end=False):
        assert all(isinstance(s, string_types) for s in sentences)
        assert isinstance(ignore_case, bool)
        assert isinstance(error_message, string_types)

        self.sentences = list(sentences)
        self.ignore_case = ignore_case
        self.error_message = error_message
        self.move_cursor_to_end = move_cursor_to_end

        if ignore_case:
            self.sentences = {s.lower() for s in self.sentences}

    def validate(self, document):
        if document.text not in self.sentences:
            if self.move_cursor_to_end:
                index = len(document.text)
            else:
                index = 0

            raise ValidationError(cursor_position=index,
                                  message=self.error_message)



"""
Command line layout definitions
-------------------------------

The layout of a command line interface is defined by a Container instance.
There are two main groups of classes here. Containers and controls:

- A container can contain other containers or controls, it can have multiple
  children and it decides about the dimensions.
- A control is responsible for rendering the actual content to a screen.
  A control can propose some dimensions, but it's the container who decides
  about the dimensions -- or when the control consumes more space -- which part
  of the control will be visible.


Container classes::

    - Container (Abstract base class)
       |- HSplit (Horizontal split)
       |- VSplit (Vertical split)
       |- VSplit (Vertical split)
       |- FloatContainer (Container which can also contain menus and other floats)
       `- Window (Container which contains one actual control

Control classes::

    - UIControl (Abstract base class)
       |- TokenListControl (Renders a simple list of tokens)
       |- FillControl (Fills control with one token/character.)
       `- BufferControl (Renders an input buffer.)


Usually, you end up wrapping every control inside a `Window` object, because
that's the only way to render it in a layout.

There are some prepared toolbars which are ready to use::

- SystemToolbar (Shows the 'system' input buffer, for entering system commands.)
- ArgToolbar (Shows the input 'arg', for repetition of input commands.)
- SearchToolbar (Shows the 'search' input buffer, for incremental search.)
- CompletionsToolbar (Shows the completions of the current buffer.)
- ValidationToolbar (Shows validation errors of the current buffer.)

And one prepared menu:

- CompletionsMenu

"""
from __future__ import unicode_literals

from .containers import Float, FloatContainer, HSplit, VSplit, Window, ConditionalContainer
from .controls import TokenListControl, FillControl, BufferControl



from __future__ import unicode_literals

from prompt_toolkit.key_binding.input_processor import InputProcessor, KeyPress
from prompt_toolkit.key_binding.registry import Registry
from prompt_toolkit.keys import Keys

import pytest


class Handlers(object):

    def __init__(self):
        self.called = []

    def __getattr__(self, name):
        def func(event):
            self.called.append(name)
        return func


@pytest.fixture
def handlers():
    return Handlers()


@pytest.fixture
def registry(handlers):
    registry = Registry()
    registry.add_binding(
        Keys.ControlX, Keys.ControlC)(handlers.controlx_controlc)
    registry.add_binding(Keys.ControlX)(handlers.control_x)
    registry.add_binding(Keys.ControlD)(handlers.control_d)
    registry.add_binding(
        Keys.ControlSquareClose, Keys.Any)(handlers.control_square_close_any)

    return registry


@pytest.fixture
def processor(registry):
    return InputProcessor(registry, lambda: None)


def test_feed_simple(processor, handlers):
    processor.feed(KeyPress(Keys.ControlX, '\x18'))
    processor.feed(KeyPress(Keys.ControlC, '\x03'))
    processor.process_keys()

    assert handlers.called == ['controlx_controlc']


def test_feed_several(processor, handlers):
    # First an unknown key first.
    processor.feed(KeyPress(Keys.ControlQ, ''))
    processor.process_keys()

    assert handlers.called == []

    # Followed by a know key sequence.
    processor.feed(KeyPress(Keys.ControlX, ''))
    processor.feed(KeyPress(Keys.ControlC, ''))
    processor.process_keys()

    assert handlers.called == ['controlx_controlc']

    # Followed by another unknown sequence.
    processor.feed(KeyPress(Keys.ControlR, ''))
    processor.feed(KeyPress(Keys.ControlS, ''))

    # Followed again by a know key sequence.
    processor.feed(KeyPress(Keys.ControlD, ''))
    processor.process_keys()

    assert handlers.called == ['controlx_controlc', 'control_d']


def test_control_square_closed_any(processor, handlers):
    processor.feed(KeyPress(Keys.ControlSquareClose, ''))
    processor.feed(KeyPress('C', 'C'))
    processor.process_keys()

    assert handlers.called == ['control_square_close_any']


def test_common_prefix(processor, handlers):
    # Sending Control_X should not yet do anything, because there is
    # another sequence starting with that as well.
    processor.feed(KeyPress(Keys.ControlX, ''))
    processor.process_keys()

    assert handlers.called == []

    # When another key is pressed, we know that we did not meant the longer
    # "ControlX ControlC" sequence and the callbacks are called.
    processor.feed(KeyPress(Keys.ControlD, ''))
    processor.process_keys()

    assert handlers.called == ['control_x', 'control_d']
